import { world, system } from "@minecraft/server";

const moonPhaseList = [
    "0 (Full Moon)",
    "1 (Waning Gibbous)",
    "2 (First Quarter)",
    "3 (Waning Crescent)",
    "4 (New Moon)",
    "5 (Waxing Crescent)",
    "6 (Last Quarter)",
    "7 (Waxing Gibbous)"
];

var lastTick = Date.now();
var tps = 20;
var timeArray = [];
system.runInterval(() => {
    if (timeArray.length === 20) timeArray.shift();
    timeArray.push(Math.round(1000 / (Date.now() - lastTick) * 100) / 100);
    tps = Math.round(timeArray.reduce((a, b) => a + b) / timeArray.length);
    lastTick = Date.now();
})

function getElapsedTime(joinTime) {
    const ticks = system.currentTick - joinTime;
    const secs = Math.floor((ticks / 20) % 60).toString().padStart(2, "0");
    const mins = Math.floor((ticks / 1200) % 60).toString().padStart(2, "0");
    const hours = Math.floor(ticks / 72000).toString().padStart(2, "0");
    return hours + ":" + mins + ":" + secs;
}

function getDirection(angle) {
    if(angle < 0) angle = 360 - Math.abs(angle);
    if((225 < angle) && (angle <= 315)) {
        return "East";
    } else if((45 <= angle) && (angle < 135)) {
        return "West";
    } else if ((135 < angle) && (angle <= 225)) {
        return "North";
    } else {
        return "South";
    }
}

function getChunkLoc(location) {
    const chunkX = Math.floor(location.x/16);
    const chunkZ = Math.floor(location.z/16);
    const locInChunkX = Math.floor(location.x - 16 * chunkX);
    const locInChunkZ = Math.floor(location.z - 16 * chunkZ);
    return `${chunkX}, ${chunkZ} (${locInChunkX}, ${locInChunkZ})`;
}

world.afterEvents.playerSpawn.subscribe((e) => {
    if(e.initialSpawn == false) return;
    const joinTime = system.currentTick;
    const player = e.player;
    
    // data store variables
    var biome;
    var lightLevel;
    var difficulty;
    var inVillage;
    var isUnderground;

    // data check variables
    var elTime;
    var chunkLoc;
    var entityCount;
    var dimension;
    var direction;
    var weather;
    var dayCount;
    var moonPhase;
    var preTps;
    var preLightLevel;
    var preDifficulty;
    var preBiome;
    var preInVillage;
    var preIsUnderground;

    // data event recievers
    system.afterEvents.scriptEventReceive.subscribe((e) => {
        if(e.sourceEntity != player || !e.id.startsWith("star:biome")) return;
        biome = e.message;
    });
    
    world.beforeEvents.dataDrivenEntityTriggerEvent.subscribe((e) => {
        if(e.entity != player || !e.id.startsWith("star")) return;
        const id = e.id.slice(5, 9);
        const value = e.id.slice(10);
        
        if(id == "0001") {
            lightLevel = value;
        } else if(id == "0010") {
            difficulty = value;
        } else if(id == "0011") {
            inVillage = value;
        } else if(id == "0100") {
            isUnderground = value;
        }
    });
    
    // data transfer
    var i2 = 1;
    var i3 = 1;
    var i4 = 1;
    system.runInterval(() => {
        // tick 1
        if(!player.isValid()) return;
       player.runCommandAsync(`title @s title &&&!Angle of Attack: ${Math.round(player.getRotation().x*(-1))}º`);

        // tick 2
        system.runTimeout(() => {
            if(direction != getDirection(player.getRotation().y)) {
                direction = getDirection(player.getRotation().y);
                player.runCommandAsync(`title @s title &&!&Facing Direction: ${direction}`);
            } else if(chunkLoc != getChunkLoc(player.location)) {
                chunkLoc = getChunkLoc(player.location);
                player.runCommandAsync(`title @s title !!!!Chunk Loc: ${chunkLoc}`);
            } else if(entityCount != player.dimension.getEntities({location: player.location, maxDistance: 96}).length) {
                entityCount = player.dimension.getEntities({location: player.location, maxDistance: 96}).length;
                player.runCommandAsync(`title @s title !!&!Entity Count: ${entityCount}`);
            } else {
                switch(i2) {
                    case 1:
                        i2 = 2;
                        player.runCommandAsync(`title @s title !!&!Entity Count: ${entityCount}`);
                        break;
                    case 2:
                        i2 = 3;
                        player.runCommandAsync(`title @s title &&!&Facing Direction: ${direction}`);
                        break;
                    case 3:
                        i2 = 1;
                        player.runCommandAsync(`title @s title !!!!Chunk Loc: ${chunkLoc}`);
                }
            }
        }, 1);

        // tick 3
        system.runTimeout(() => {
            if(preTps != tps) {
                preTps = tps;
                player.runCommandAsync(`title @s title &&!!TPS: ${tps}`);
            } else if(preLightLevel != lightLevel) {
                preLightLevel = lightLevel;
                player.runCommandAsync(`title @s title &!&&Light level: ${lightLevel}`);
            } else {
                switch(i3) {
                    case 1:
                        i3 = 2;
                        player.runCommandAsync(`title @s title &&!!TPS: ${tps}`);
                        break;
                    case 2:
                        i3 = 1;
                        player.runCommandAsync(`title @s title &!&&Light level: ${lightLevel}`);
                }
            }
        }, 2);

        // tick 4
        system.runTimeout(() => {
            if(elTime != getElapsedTime(joinTime)) {
                elTime = getElapsedTime(joinTime);
                player.runCommandAsync(`title @s title !!!&Elapsed Time: ${elTime}`);
            } else if(dimension != player.dimension.id) {
                dimension = player.dimension.id;
                player.runCommandAsync(`title @s title &!&!Dimension: ${dimension}`);
            } else if(weather != player.dimension.getWeather()) {
                weather = player.dimension.getWeather();
                player.runCommandAsync(`title @s title &!!&Weather: ${weather}`);
            } else if(dayCount != world.getDay()) {
                dayCount = world.getDay();
                player.runCommandAsync(`title @s title &!!!Day count: ${dayCount}`);
            } else if(moonPhase != world.getMoonPhase()) {
                moonPhase = world.getMoonPhase();
                player.runCommandAsync(`title @s title !&&&Moon phase: ${moonPhaseList[moonPhase]}`);
            } else if(preBiome != biome) {
                preBiome = biome;
                player.runCommandAsync(`title @s title !&&!Biome: ${biome}`);
            } else if(preInVillage != inVillage) {
                preInVillage = inVillage;
                player.runCommandAsync(`title @s title !&!&In village?: ${inVillage}`);
            } else if(preIsUnderground != isUnderground) {
                preIsUnderground = isUnderground;
                player.runCommandAsync(`title @s title !&!!Is underground?: ${isUnderground}`);
            } else if(preDifficulty != difficulty) {
                preDifficulty = difficulty;
                player.runCommandAsync(`title @s title !!&&Difficulty: ${difficulty}`);
            } else {
                switch(i4) {
                    case 1:
                        i4 = 2;
                        player.runCommandAsync(`title @s title &!&!Dimension: ${dimension}`);
                        break;
                    case 2:
                        i4 = 3;
                        player.runCommandAsync(`title @s title &!!&Weather: ${weather}`);
                        break;
                    case 3:
                        i4 = 4;
                        player.runCommandAsync(`title @s title &!!!Day count: ${dayCount}`);
                        break;
                    case 4:
                        i4 = 5;
                        player.runCommandAsync(`title @s title !&&&Moon phase: ${moonPhaseList[moonPhase]}`);
                        break;
                    case 5:
                        i4 = 6;
                        player.runCommandAsync(`title @s title !&&!Biome: ${biome}`);
                        break;
                    case 6:
                        i4 = 7;
                        player.runCommandAsync(`title @s title !&!&In village?: ${inVillage}`);
                        break;
                    case 7:
                        i4 = 8;
                        player.runCommandAsync(`title @s title !&!!Is underground?: ${isUnderground}`);
                        break;
                    case 8:
                        i4 = 9;
                        player.runCommandAsync(`title @s title !!&&Difficulty: ${difficulty}`);
                        break;
                    case 9:
                        i4 = 1;
                        player.runCommandAsync(`title @s title !!!&Elapsed Time: ${elTime}`);
                }
            }
        }, 3);
    }, 4);
});
